@@ -90,6 +90,8 @@ def test_initialize_from_fits_with_data_in_different_extension(tmpdir):
     ccd = CCDData.read(filename, unit='adu')
     # ccd should pick up the unit adu from the fits header...did it?
     np.testing.assert_array_equal(ccd.data, fake_img)
+    # check that the header is the combined header
+    assert hdu1.header + hdu2.header == ccd.header
 
 
 def test_initialize_from_fits_with_extension(tmpdir):
@@ -408,6 +410,29 @@ def test_arithmetic_overload_ccddata_operand(ccd_data):
                                expected_uncertainty)
 
 
+def test_arithmetic_overload_differing_units():
+    a = np.array([1, 2, 3]) * u.m
+    b = np.array([1, 2, 3]) * u.cm
+    ccddata = CCDData(a)
+
+    # TODO: Could also be parametrized.
+    res = ccddata.add(b)
+    np.testing.assert_array_almost_equal(res.data, np.add(a, b).value)
+    assert res.unit == np.add(a, b).unit
+
+    res = ccddata.subtract(b)
+    np.testing.assert_array_almost_equal(res.data, np.subtract(a, b).value)
+    assert res.unit == np.subtract(a, b).unit
+
+    res = ccddata.multiply(b)
+    np.testing.assert_array_almost_equal(res.data, np.multiply(a, b).value)
+    assert res.unit == np.multiply(a, b).unit
+
+    res = ccddata.divide(b)
+    np.testing.assert_array_almost_equal(res.data, np.divide(a, b).value)
+    assert res.unit == np.divide(a, b).unit
+
+
 def test_ccddata_header_does_not_corrupt_fits(ccd_data, tmpdir):
     # This test is for the problem described in astropy/ccdproc#165
     # The issue comes up when a long FITS keyword value is in a header
@@ -542,5 +567,90 @@ def test_wcs_attribute(ccd_data, tmpdir):
 
 def test_header(ccd_data):
     a = {'Observer': 'Hubble'}
-    ccd = CCDData(ccd_data, header = a)
+    ccd = CCDData(ccd_data, header=a)
     assert ccd.meta == a
+
+
+def test_wcs_arithmetic(ccd_data):
+    ccd_data.wcs = 5
+    result = ccd_data.multiply(1.0)
+    assert result.wcs == 5
+
+
+@pytest.mark.parametrize('operation',
+                         ['multiply', 'divide', 'add', 'subtract'])
+def test_wcs_arithmetic_ccd(ccd_data, operation):
+    ccd_data2 = ccd_data.copy()
+    ccd_data.wcs = 5
+    method = ccd_data.__getattribute__(operation)
+    result = method(ccd_data2)
+    assert result.wcs == ccd_data.wcs
+    assert ccd_data2.wcs is None
+
+@pytest.mark.parametrize('operation',
+                         ['multiply', 'divide', 'add', 'subtract'])
+def test_mask_arithmetic_ccd(ccd_data, operation):
+    ccd_data2 = ccd_data.copy()
+    ccd_data.mask = (ccd_data.data > 0)
+    method = ccd_data.__getattribute__(operation)
+    result = method(ccd_data2)
+    np.testing.assert_equal(result.mask, ccd_data.mask)
+
+
+
+def test_write_read_multiextensionfits_mask_default(ccd_data, tmpdir):
+    # Test that if a mask is present the mask is saved and loaded by default.
+    ccd_data.mask = ccd_data.data > 10
+    filename = tmpdir.join('afile.fits').strpath
+    ccd_data.write(filename)
+    ccd_after = CCDData.read(filename)
+    assert ccd_after.mask is not None
+    np.testing.assert_array_equal(ccd_data.mask, ccd_after.mask)
+
+
+def test_write_read_multiextensionfits_uncertainty_default(ccd_data, tmpdir):
+    # Test that if a uncertainty is present it is saved and loaded by default.
+    ccd_data.uncertainty = StdDevUncertainty(ccd_data.data * 10)
+    filename = tmpdir.join('afile.fits').strpath
+    ccd_data.write(filename)
+    ccd_after = CCDData.read(filename)
+    assert ccd_after.uncertainty is not None
+    np.testing.assert_array_equal(ccd_data.uncertainty.array,
+                                  ccd_after.uncertainty.array)
+
+
+def test_write_read_multiextensionfits_not(ccd_data, tmpdir):
+    # Test that writing mask and uncertainty can be disabled
+    ccd_data.mask = ccd_data.data > 10
+    ccd_data.uncertainty = StdDevUncertainty(ccd_data.data * 10)
+    filename = tmpdir.join('afile.fits').strpath
+    ccd_data.write(filename, hdu_mask=None, hdu_uncertainty=None)
+    ccd_after = CCDData.read(filename)
+    assert ccd_after.uncertainty is None
+    assert ccd_after.mask is None
+
+
+def test_write_read_multiextensionfits_custom_ext_names(ccd_data, tmpdir):
+    # Test writing mask, uncertainty in another extension than default
+    ccd_data.mask = ccd_data.data > 10
+    ccd_data.uncertainty = StdDevUncertainty(ccd_data.data * 10)
+    filename = tmpdir.join('afile.fits').strpath
+    ccd_data.write(filename, hdu_mask='Fun', hdu_uncertainty='NoFun')
+
+    # Try reading with defaults extension names
+    ccd_after = CCDData.read(filename)
+    assert ccd_after.uncertainty is None
+    assert ccd_after.mask is None
+
+    # Try reading with custom extension names
+    ccd_after = CCDData.read(filename, hdu_mask='Fun', hdu_uncertainty='NoFun')
+    assert ccd_after.uncertainty is not None
+    assert ccd_after.mask is not None
+    np.testing.assert_array_equal(ccd_data.mask, ccd_after.mask)
+    np.testing.assert_array_equal(ccd_data.uncertainty.array,
+                                  ccd_after.uncertainty.array)
+
+def test_wcs(ccd_data):
+    ccd_data.wcs = 5
+    assert ccd_data.wcs == 5
+
