@@ -9,7 +9,9 @@ import astropy.units as u
 from astropy.stats import median_absolute_deviation as mad
 
 from astropy.tests.helper import pytest
-
+from astropy.utils.data import get_pkg_data_filename
+from astropy.utils.compat import NUMPY_LT_1_9
+from astropy.wcs import WCS
 
 from ..ccddata import CCDData
 from ..combiner import *
@@ -183,18 +185,18 @@ def test_combiner_sigmaclip_low():
     assert c.data_arr[5].mask.all()
 
 
-#test that the average combination works and returns a ccddata object
+#test that the median combination works and returns a ccddata object
 def test_combiner_median(ccd_data):
     ccd_list = [ccd_data, ccd_data, ccd_data]
     c = Combiner(ccd_list)
-    ccd = c.average_combine()
+    ccd = c.median_combine()
     assert isinstance(ccd, CCDData)
     assert ccd.shape == (100, 100)
     assert ccd.unit == u.adu
     assert ccd.meta['NCOMBINE'] == len(ccd_list)
 
 
-#test that the median combination works and returns a ccddata object
+#test that the average combination works and returns a ccddata object
 def test_combiner_average(ccd_data):
     ccd_list = [ccd_data, ccd_data, ccd_data]
     c = Combiner(ccd_list)
@@ -256,7 +258,7 @@ def test_combiner_scaling_fails(ccd_data):
 
 
 #test data combined with mask is created correctly
-def test_combiner_mask_media(ccd_data):
+def test_combiner_mask_median(ccd_data):
     data = np.zeros((10, 10))
     data[5, 5] = 1
     mask = (data == 0)
@@ -268,3 +270,116 @@ def test_combiner_mask_media(ccd_data):
     assert ccd.data[5, 5] == 1
     assert ccd.mask[0, 0]
     assert not ccd.mask[5, 5]
+
+#test combiner convenience function reads fits file and combine as expected
+def test_combine_average_fitsimages():
+    fitsfile = get_pkg_data_filename('data/a8280271.fits')
+    ccd = CCDData.read(fitsfile, unit=u.adu)
+    ccd_list = [ccd]*3
+    c = Combiner(ccd_list)
+    ccd_by_combiner = c.average_combine()
+
+    fitsfilename_list = [fitsfile]*3
+    avgccd = combine(fitsfilename_list, output_file=None, method='average', unit=u.adu)
+    # averaging same fits images should give back same fits image
+    np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data)
+
+
+#test combiner convenience function works with list of ccddata objects
+def test_combine_average_ccddata():
+    fitsfile = get_pkg_data_filename('data/a8280271.fits')
+    ccd = CCDData.read(fitsfile, unit=u.adu)
+    ccd_list = [ccd]*3
+    c = Combiner(ccd_list)
+    ccd_by_combiner = c.average_combine()
+
+    avgccd = combine(ccd_list,output_file=None, method='average', unit=u.adu)
+    # averaging same ccdData should give back same images
+    np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data)
+
+
+#test combiner convenience function reads fits file and
+# and combine as expected when asked to run in limited memory
+def test_combine_limitedmem_fitsimages():
+    fitsfile = get_pkg_data_filename('data/a8280271.fits')
+    ccd = CCDData.read(fitsfile, unit=u.adu)
+    ccd_list = [ccd]*5
+    c = Combiner(ccd_list)
+    ccd_by_combiner = c.average_combine()
+
+    fitsfilename_list = [fitsfile]*5
+    avgccd = combine(fitsfilename_list,output_file=None, method='average', mem_limit=1e6,
+                     unit=u.adu)
+    # averaging same ccdData should give back same images
+    np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data)
+
+
+#test combiner convenience function reads fits file and
+# and combine as expected when asked to run in limited memory with scaling
+@pytest.mark.xfail(NUMPY_LT_1_9,
+                   reason="numpy < 1.9 loses precision in np.ma.average")
+def test_combine_limitedmem_scale_fitsimages():
+    fitsfile = get_pkg_data_filename('data/a8280271.fits')
+    ccd = CCDData.read(fitsfile, unit=u.adu)
+    ccd_list = [ccd]*5
+    c = Combiner(ccd_list)
+    # scale each array to the mean of the first image
+    scale_by_mean = lambda x: ccd.data.mean()/np.ma.average(x)
+    c.scaling = scale_by_mean
+    ccd_by_combiner = c.average_combine()
+
+    fitsfilename_list = [fitsfile]*5
+    avgccd = combine(fitsfilename_list,output_file=None, method='average',mem_limit=1e6,
+                     scale=scale_by_mean, unit=u.adu)
+
+    np.testing.assert_array_almost_equal(avgccd.data, ccd_by_combiner.data, decimal = 4)
+
+
+#test the optional uncertainty function in average_combine
+def test_average_combine_uncertainty(ccd_data):
+    ccd_list = [ccd_data, ccd_data, ccd_data]
+    c = Combiner(ccd_list)
+    ccd = c.average_combine(uncertainty_func=np.sum)
+    uncert_ref = np.sum(c.data_arr, 0) / np.sqrt(3)
+    np.testing.assert_array_equal(ccd.uncertainty.array, uncert_ref)
+
+#test the optional uncertainty function in median_combine
+def test_median_combine_uncertainty(ccd_data):
+    ccd_list = [ccd_data, ccd_data, ccd_data]
+    c = Combiner(ccd_list)
+    ccd = c.median_combine(uncertainty_func=np.sum)
+    uncert_ref = np.sum(c.data_arr, 0) / np.sqrt(3)
+    np.testing.assert_array_equal(ccd.uncertainty.array, uncert_ref)
+
+
+# test resulting uncertainty is corrected for the number of images
+def test_combiner_uncertainty_average():
+    ccd_list = [CCDData(np.ones((10, 10)), unit=u.adu),
+                CCDData(np.ones((10, 10))*2, unit=u.adu)]
+    c = Combiner(ccd_list)
+    ccd = c.average_combine()
+    # Just the standard deviation of ccd data.
+    ref_uncertainty = np.ones((10, 10)) / 2
+    # Correction because we combined two images.
+    ref_uncertainty /= np.sqrt(2)
+    np.testing.assert_array_almost_equal(ccd.uncertainty.array,
+                                         ref_uncertainty)
+
+
+# test resulting uncertainty is corrected for the number of images (with mask)
+def test_combiner_uncertainty_average_mask():
+    mask = np.zeros((10, 10), dtype=np.bool_)
+    mask[5, 5] = True
+    ccd_with_mask = CCDData(np.ones((10, 10)), unit=u.adu, mask=mask)
+    ccd_list = [ccd_with_mask,
+                CCDData(np.ones((10, 10))*2, unit=u.adu),
+                CCDData(np.ones((10, 10))*3, unit=u.adu)]
+    c = Combiner(ccd_list)
+    ccd = c.average_combine()
+    # Just the standard deviation of ccd data.
+    ref_uncertainty = np.ones((10, 10)) * np.std([1, 2, 3])
+    # Correction because we combined two images.
+    ref_uncertainty /= np.sqrt(3)
+    ref_uncertainty[5, 5] = np.std([2, 3]) / np.sqrt(2)
+    np.testing.assert_array_almost_equal(ccd.uncertainty.array,
+                                         ref_uncertainty)
